package com.ozs.service.service;

import com.ozs.service.entity.vo.BaseCameraVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.SetOperations;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author wyy
 * @subject
 * @creat 2024/3/7
 */
@Service
public class RedisService {

    @Autowired
    public RedisTemplate<String, BaseCameraVO> redisTemplate;

    // =============================common============================
    /**
     * 指定缓存失效时间
     * @param key 键
     * @param time 时间(秒)
     * @return
     */
    public boolean expire(String key, long time) {
        try {
            if (time > 0) {
                redisTemplate.expire(key, time, TimeUnit.SECONDS);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * 根据key 获取过期时间
     * @param key 键 不能为null
     * @return 时间(秒) 返回0代表为永久有效
     */
    public long getExpire(String key) {
        return redisTemplate.getExpire(key, TimeUnit.SECONDS);
    }

    /**
     * 判断key是否存在
     * @param key 键
     * @return true 存在 false不存在
     */
    public boolean hasKey(String key) {
        try {
            return redisTemplate.hasKey(key);
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    // ============================String=============================
    /**
     * 普通缓存获取
     * @param key 键
     * @return 值
     */
    public Object get(String key) {
        return key == null ? null : redisTemplate.opsForValue().get(key);
    }

    /**
     * 普通缓存放入
     * @param key 键
     * @param value 值
     * @return true成功 false失败
     */
    public boolean set(String key, BaseCameraVO value) {
        try {
            redisTemplate.opsForValue().set(key, value);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }

    }

    /**
     * 普通缓存放入并设置时间
     * @param key 键
     * @param value 值
     * @param time 时间(秒) time要大于0 如果time小于等于0 将设置无限期
     * @return true成功 false 失败
     */
    public boolean set(String key, BaseCameraVO value, long time) {
        try {
            if (time > 0) {
                redisTemplate.opsForValue().set(key, value, time, TimeUnit.SECONDS);
            } else {
                set(key, value);
            }
            return true;
        } catch (Exception e) {
            e.printStackTrace();
            return false;
        }
    }

    // 将对象添加到set集合
    public void addToSet(String key, BaseCameraVO object) {
        redisTemplate.opsForSet().add(key, object);
    }

    // 将对象添加到hash集合
    public void addToHash(String key, String hashKey, BaseCameraVO vo) {
        redisTemplate.opsForHash().put(key, hashKey, vo);
    }

    // 从set集合中删除对象
    public long removeFromSet(String key, BaseCameraVO object) {
        return redisTemplate.opsForSet().remove(key, object);
    }

    // 删除set集合
    public boolean delete(String key) {
        return redisTemplate.delete(key);
    }

    // 根据KEY清空set集合
    public boolean removeBySetItemKey(String key, String itemKey) {
        SetOperations<String, BaseCameraVO> setOperations = redisTemplate.opsForSet();
        List<BaseCameraVO> _filters = setOperations.members(key).stream()
                .filter(f -> f.getCameraCode().equalsIgnoreCase(itemKey)).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(_filters)) {
            for (BaseCameraVO ele : _filters) {
                setOperations.remove(key, ele);
            }
        }
        return true;
    }

    // 根据KEY清空set集合
    public boolean clearFromSet(String key) {
        SetOperations<String, BaseCameraVO> setOperations = redisTemplate.opsForSet();
        Set<BaseCameraVO> sets = setOperations.members(key);
        for (BaseCameraVO ele : sets) {
            setOperations.remove(key, ele);
        }
        return true;
    }

    // 根据项KEY判断是否存在set集合中
    public boolean existFromSet(String key, String itemKey) {
        return !CollectionUtils.isEmpty(redisTemplate.opsForSet().members(key).stream()
                .filter(f -> f.getCameraCode().equalsIgnoreCase(itemKey)).collect(Collectors.toList()));
    }

    // 获取set集合中的所有对象
    public Set<BaseCameraVO> getSetMembers(String key) {
        return redisTemplate.opsForSet().members(key);
    }
}