package com.ozs.web.controller.shotschedule;

import cn.hutool.core.date.DateField;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUnit;
import cn.hutool.core.lang.Dict;
import cn.hutool.json.JSONUtil;
import com.ozs.common.utils.StringUtils;
import com.ozs.service.entity.vo.BaseCameraVO;
import com.ozs.service.service.RedisService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.data.redis.connection.stream.ObjectRecord;
import org.springframework.data.redis.connection.stream.RecordId;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.stream.StreamListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.LocalTime;

@Component
@Slf4j
public class CaptureStreamListener implements StreamListener<String, ObjectRecord<String, String>> {

    @Resource
    private StringRedisTemplate stringRedisTemplate;

    @Resource
    public RedisTemplate<String, String> redisTemplate;

    /**
     * 生成 runable 任务服务提供类
     */
    @Resource
    private GenPictureTaskService genPictureTaskService;

    /**
     * Redis 缓存服务提供类
     */
    @Resource
    private RedisService redisService;

    @Override
    protected void finalize() throws Throwable {
        super.finalize();
    }

    @Override
    @Async
    public void onMessage(ObjectRecord<String, String> message) {
        ShotPictureTaskExecutors.RUNNING_ATOMIC_INT.incrementAndGet();
        BaseCameraVO vo = null;
        try {
            // 消息ID
            RecordId recordId = message.getId();
            // 消息的key和value
            String value = message.getValue();
            if (StringUtils.isNotBlank(value) && value.indexOf("cameraCode") == -1) {
                stringRedisTemplate.opsForStream().acknowledge(RedisPushService.STREAM_KEY, RedisPushService.STREAM_CONSUMER_GROUP_KEY, recordId.getValue());
                stringRedisTemplate.opsForStream().delete(RedisPushService.STREAM_KEY, recordId.getValue());
                ShotPictureTaskExecutors.RUNNING_ATOMIC_INT.decrementAndGet();
                return;
            }
            Dict dataDict = JSONUtil.toBean(value, Dict.class);

            String cameraCode = dataDict.getStr("cameraCode");
            String railwayName = dataDict.getStr("railwayName");
            String installMiles = dataDict.getStr("installMiles");
            Boolean isAlarm = dataDict.getBool("isAlarm");
            String camera_type_hash = isAlarm ? ShotPictureTaskExecutors.CAMERA_ALARM_HASH : ShotPictureTaskExecutors.CAMERA_NORMAL_HASH;

            Object cacheObj = redisTemplate.opsForHash().get(camera_type_hash, cameraCode);
            vo = JSONUtil.toBean(ObjectUtils.toString(cacheObj), BaseCameraVO.class);
            vo.setStatus("running");
            if (vo.getIsAlarm() == null) {
                vo.setIsAlarm(isAlarm);
            }
            redisService.addToHash(camera_type_hash, vo.getCameraCode(), vo);

            DateTime nextExecuteTime = DateTime.now();
            if (StringUtils.isNotBlank(dataDict.getStr("nextExecuteTime"))) {
                nextExecuteTime = DateTime.of(dataDict.getStr("nextExecuteTime"), "yyyy-MM-dd HH:mm:ss");
            }
            DateTime now = DateTime.now();
            StringBuilder fileName = new StringBuilder(cameraCode);
            fileName.append("-");
            fileName.append(DateTime.now().toString("yyyyMMddHHmmss"));
            String isNormalPath = isAlarm ? "alarm" : "normal";
            String filePath = dataDict.getStr("address")+"/"+isNormalPath+"/"+railwayName+"/"+installMiles+"/"+ LocalDate.now().getYear() +"/"+ LocalDate.now().getMonthValue() +"/"+LocalDate.now().getDayOfMonth()+ "/"+ LocalTime.now().getHour()+"/" +fileName + ".jpeg";
            // 超过时间则进行抓拍
            if (now.isAfterOrEquals(nextExecuteTime) && nextExecuteTime.between(now, DateUnit.MINUTE) < 2) {
                genPictureTaskService.makePicture(cameraCode, dataDict.getStr("channel"), filePath, vo.getIsAlarm());
            }
            // 通过RedisTemplate手动确认消息
            stringRedisTemplate.opsForStream().acknowledge(RedisPushService.STREAM_KEY, RedisPushService.STREAM_CONSUMER_GROUP_KEY, recordId.getValue());
            stringRedisTemplate.opsForStream().delete(RedisPushService.STREAM_KEY, recordId.getValue());
        } catch (Exception e) {
            log.error("摄像头抓图出现异常, ", e);
        } finally {
            Boolean isAlarm = vo.getIsAlarm();
            if (isAlarm) {
                vo.setNextExecuteTime(DateTime.now().offset(DateField.SECOND, 1).toString("yyyy-MM-dd HH:mm:ss"));
            } else {
                vo.setNextExecuteTime(DateTime.now().offset(DateField.MINUTE, 1).toString("yyyy-MM-dd HH:mm:ss"));
            }
            vo.setStatus("complete");
            redisService.addToHash(isAlarm ? ShotPictureTaskExecutors.CAMERA_ALARM_HASH :
                    ShotPictureTaskExecutors.CAMERA_NORMAL_HASH, vo.getCameraCode(), vo);
            ShotPictureTaskExecutors.RUNNING_ATOMIC_INT.decrementAndGet();
        }
    }

}
